@@ -6,20 +6,16 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID("$Id: chkname.c,v 1.6 2002/01/10 13:04:34 kloczek Exp $")
-
+RCSID ("$Id: chkname.c,v 1.8 2003/05/12 05:29:14 kloczek Exp $")
 #include <ctype.h>
 #include "defines.h"
 #include "chkname.h"
-
 #if HAVE_UTMPX_H
 #include <utmpx.h>
 #else
 #include <utmp.h>
 #endif
-
-static int
-good_name(const char *name)
+static int good_name (const char *name)
 {
 	/*
 	 * User/group names must match [a-z_][a-z0-9_-]*
@@ -29,17 +25,16 @@ good_name(const char *name)
 
 	while (*++name) {
 		if (!((*name >= 'a' && *name <= 'z') ||
-		    (*name >= '0' && *name <= '9') ||
-		    *name == '_' || *name == '-' ||
-		    (*name == '$' && *(name+1) == NULL)))
+		      (*name >= '0' && *name <= '9') ||
+		      *name == '_' || *name == '-' ||
+		      (*name == '$' && *(name + 1) == '\0')))
 			return 0;
 	}
 
 	return 1;
 }
 
-int
-check_user_name(const char *name)
+int check_user_name (const char *name)
 {
 #if HAVE_UTMPX_H
 	struct utmpx ut;
@@ -51,21 +46,20 @@ check_user_name(const char *name)
 	 * User names are limited by whatever utmp can
 	 * handle (usually max 8 characters).
 	 */
-	if (strlen(name) > sizeof(ut.ut_user))
+	if (strlen (name) > sizeof (ut.ut_user))
 		return 0;
 
-	return good_name(name);
+	return good_name (name);
 }
 
-int
-check_group_name(const char *name)
+int check_group_name (const char *name)
 {
 	/*
 	 * Arbitrary limit for group names - max 16
 	 * characters (same as on HP-UX 10).
 	 */
-	if (strlen(name) > 16)
+	if (strlen (name) > 16)
 		return 0;
 
-	return good_name(name);
+	return good_name (name);
 }
