@@ -41,7 +41,7 @@
 #include <time.h>
 #include "rcsid.h"
 
-RCSID ("$Id: isexpired.c,v 1.11 2003/05/03 16:14:33 kloczek Exp $")
+RCSID ("$Id: isexpired.c,v 1.14 2005/05/25 19:31:50 kloczek Exp $")
 
 /*
  * isexpired - determine if account is expired yet
@@ -49,9 +49,7 @@ RCSID ("$Id: isexpired.c,v 1.11 2003/05/03 16:14:33 kloczek Exp $")
  *	isexpired calculates the expiration date based on the
  *	password expiration criteria.
  */
-     /*ARGSUSED*/
-#ifdef	SHADOWPWD
-int isexpired (const struct passwd *pw, const struct spwd *sp)
+ /*ARGSUSED*/ int isexpired (const struct passwd *pw, const struct spwd *sp)
 {
 	long now;
 
@@ -77,8 +75,7 @@ int isexpired (const struct passwd *pw, const struct spwd *sp)
 	 * if /etc/shadow doesn't exist, getspnam() still succeeds and
 	 * returns sp_lstchg==0 (must change password) instead of -1!
 	 */
-	if (sp->sp_lstchg == 0
-	    && !strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING))
+	if (sp->sp_lstchg == 0 && !strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING))
 		return 1;
 
 	if (sp->sp_lstchg > 0 && sp->sp_max >= 0 && sp->sp_inact >= 0 &&
@@ -105,4 +102,3 @@ int isexpired (const struct passwd *pw, const struct spwd *sp)
 		return 1;
 	return 0;
 }
-#endif				/* SHADOWPWD */
