@@ -30,30 +30,30 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: chpasswd.c,v 1.19 2004/10/11 06:26:40 kloczek Exp $")
-#include <stdio.h>
-#include "prototypes.h"
-#include "defines.h"
-#include <pwd.h>
+RCSID (PKG_VER "$Id: chpasswd.c,v 1.26 2005/05/25 19:31:51 kloczek Exp $")
 #include <fcntl.h>
-#include "pwio.h"
-#ifdef	SHADOWPWD
-#include "shadowio.h"
-#endif
-#include "nscd.h"
+#include <getopt.h>
+#include <pwd.h>
+#include <stdio.h>
+#include <stdlib.h>
 #ifdef USE_PAM
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
 #include <pwd.h>
 #endif				/* USE_PAM */
+#include "prototypes.h"
+#include "defines.h"
+#include "pwio.h"
+#include "nscd.h"
+#include "shadowio.h"
+/*
+ * Global variables
+ */
 static char *Prog;
 static int eflg = 0;
+static int md5flg = 0;
 
-#ifdef SHADOWPWD
 static int is_shadow_pwd;
-#endif
-
-extern char *l64a ();
 
 /* local function prototypes */
 static void usage (void);
@@ -64,7 +64,13 @@ static void usage (void);
 
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: %s [-e]\n"), Prog);
+	fprintf (stderr, _("Usage: chpasswd [options]\n"
+			   "\n"
+			   "Options:\n"
+			   "  -e, --encrypted	supplied passwords are encrypted\n"
+			   "  -h, --help		display this help message and exit\n"
+			   "  -m, --md5		use MD5 encryption instead DES when the supplied\n"
+			   "			passwords are not encrypted\n"));
 	exit (1);
 }
 
@@ -82,10 +88,9 @@ int main (int argc, char **argv)
 	char *newpwd;
 	char *cp;
 
-#ifdef	SHADOWPWD
 	const struct spwd *sp;
 	struct spwd newsp;
-#endif
+
 	const struct passwd *pw;
 	struct passwd newpw;
 	int errors = 0;
@@ -105,11 +110,38 @@ int main (int argc, char **argv)
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
-	/* XXX - use getopt() */
-	if (!(argc == 1 || (argc == 2 && !strcmp(argv[1], "-e"))))
-		usage();
-	if (argc == 2)
-		eflg = 1;
+	{
+		int option_index = 0;
+		int c;
+		static struct option long_options[] = {
+			{"encrypted", no_argument, NULL, 'e'},
+			{"help", no_argument, NULL, 'h'},
+			{"md5", no_argument, NULL, 'm'},
+			{NULL, 0, NULL, '\0'}
+		};
+
+		while ((c =
+			getopt_long (argc, argv, "ehm", long_options,
+				     &option_index)) != -1) {
+			switch (c) {
+			case 'e':
+				eflg = 1;
+				break;
+			case 'h':
+				usage ();
+				break;
+			case 'm':
+				md5flg = 1;
+				break;
+			case 0:
+				/* long option */
+				break;
+			default:
+				usage ();
+				break;
+			}
+		}
+	}
 
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
@@ -120,8 +152,7 @@ int main (int argc, char **argv)
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval =
-		    pam_start ("chpasswd", pampw->pw_name, &conv, &pamh);
+		retval = pam_start ("chpasswd", pampw->pw_name, &conv, &pamh);
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -139,8 +170,7 @@ int main (int argc, char **argv)
 	}
 
 	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM authentication failed\n"),
-			 Prog);
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (1);
 	}
 #endif				/* USE_PAM */
@@ -151,17 +181,15 @@ int main (int argc, char **argv)
 	 */
 
 	if (!pw_lock ()) {
-		fprintf (stderr, _("%s: can't lock password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: can't lock password file\n"), Prog);
 		exit (1);
 	}
 	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("%s: can't open password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: can't open password file\n"), Prog);
 		pw_unlock ();
 		exit (1);
 	}
-#ifdef SHADOWPWD
+
 	is_shadow_pwd = spw_file_present ();
 	if (is_shadow_pwd) {
 		if (!spw_lock ()) {
@@ -178,7 +206,6 @@ int main (int argc, char **argv)
 			exit (1);
 		}
 	}
-#endif
 
 	/*
 	 * Read each line, separating the user name from the password. The
@@ -220,8 +247,15 @@ int main (int argc, char **argv)
 			continue;
 		}
 		newpwd = cp;
-		if (!eflg)
-			cp = pw_encrypt (newpwd, crypt_make_salt ());
+		if (!eflg) {
+			if (md5flg) {
+				char salt[12] = "$1$";
+
+				strcat (salt, crypt_make_salt ());
+				cp = pw_encrypt (newpwd, salt);
+			} else
+				cp = pw_encrypt (newpwd, crypt_make_salt ());
+		}
 
 		/*
 		 * Get the password file entry for this user. The user must
@@ -236,12 +270,10 @@ int main (int argc, char **argv)
 			errors++;
 			continue;
 		}
-#ifdef SHADOWPWD
 		if (is_shadow_pwd)
 			sp = spw_locate (name);
 		else
 			sp = NULL;
-#endif
 
 		/*
 		 * The freshly encrypted new password is merged into the
@@ -249,14 +281,11 @@ int main (int argc, char **argv)
 		 * date is set to the current date.
 		 */
 
-#ifdef SHADOWPWD
 		if (sp) {
 			newsp = *sp;
 			newsp.sp_pwdp = cp;
 			newsp.sp_lstchg = now;
-		} else
-#endif
-		{
+		} else {
 			newpw = *pw;
 			newpw.pw_passwd = cp;
 		}
@@ -267,11 +296,9 @@ int main (int argc, char **argv)
 		 * other entries have been updated as well.
 		 */
 
-#ifdef SHADOWPWD
 		if (sp)
 			ok = spw_update (&newsp);
 		else
-#endif
 			ok = pw_update (&newpw);
 
 		if (!ok) {
@@ -295,28 +322,22 @@ int main (int argc, char **argv)
 	if (errors) {
 		fprintf (stderr,
 			 _("%s: error detected, changes ignored\n"), Prog);
-#ifdef SHADOWPWD
 		if (is_shadow_pwd)
 			spw_unlock ();
-#endif
 		pw_unlock ();
 		exit (1);
 	}
-#ifdef SHADOWPWD
 	if (is_shadow_pwd) {
 		if (!spw_close ()) {
 			fprintf (stderr,
-				 _("%s: error updating shadow file\n"),
-				 Prog);
+				 _("%s: error updating shadow file\n"), Prog);
 			pw_unlock ();
 			exit (1);
 		}
 		spw_unlock ();
 	}
-#endif
 	if (!pw_close ()) {
-		fprintf (stderr, _("%s: error updating password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error updating password file\n"), Prog);
 		exit (1);
 	}
 
