@@ -30,12 +30,17 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: groupadd.c,v 1.29 2004/10/11 06:26:40 kloczek Exp $")
+RCSID (PKG_VER "$Id: groupadd.c,v 1.37 2005/07/18 13:17:45 kloczek Exp $")
 #include <sys/types.h>
 #include <stdio.h>
 #include <grp.h>
 #include <ctype.h>
 #include <fcntl.h>
+#ifdef USE_PAM
+#include <security/pam_appl.h>
+#include <security/pam_misc.h>
+#include <pwd.h>
+#endif				/* USE_PAM */
 #include "defines.h"
 #include "prototypes.h"
 #include "chkname.h"
@@ -44,11 +49,6 @@ RCSID (PKG_VER "$Id: groupadd.c,v 1.29 2004/10/11 06:26:40 kloczek Exp $")
 #include "nscd.h"
 #ifdef	SHADOWGRP
 #include "sgroupio.h"
-#ifdef USE_PAM
-#include <security/pam_appl.h>
-#include <security/pam_misc.h>
-#include <pwd.h>
-#endif				/* USE_PAM */
 static int is_shadow_grp;
 #endif
 
@@ -73,11 +73,6 @@ static int oflg = 0;		/* permit non-unique group ID to be specified with -g */
 static int gflg = 0;		/* ID value for the new group */
 static int fflg = 0;		/* if group already exists, do nothing and exit(0) */
 
-#ifdef	NDBM
-extern int gr_dbm_mode;
-extern int sg_dbm_mode;
-#endif
-
 /* local function prototypes */
 static void usage (void);
 static void new_grent (struct group *);
@@ -154,7 +149,6 @@ static void grp_update (void)
 	/*
 	 * Create the initial entries for this new group.
 	 */
-
 	new_grent (&grp);
 #ifdef	SHADOWGRP
 	new_sgent (&sgrp);
@@ -163,52 +157,20 @@ static void grp_update (void)
 	/*
 	 * Write out the new group file entry.
 	 */
-
 	if (!gr_update (&grp)) {
-		fprintf (stderr, _("%s: error adding new group entry\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error adding new group entry\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
-#ifdef	NDBM
-
-	/*
-	 * Update the DBM group file with the new entry as well.
-	 */
-
-	if (gr_dbm_present () && !gr_dbm_update (&grp)) {
-		fprintf (stderr, _("%s: cannot add new dbm group entry\n"),
-			 Prog);
-		fail_exit (E_GRP_UPDATE);
-	}
-	endgrent ();
-#endif				/* NDBM */
-
 #ifdef	SHADOWGRP
-
 	/*
 	 * Write out the new shadow group entries as well.
 	 */
-
 	if (is_shadow_grp && !sgr_update (&sgrp)) {
-		fprintf (stderr, _("%s: error adding new group entry\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error adding new group entry\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
-#ifdef	NDBM
-
-	/*
-	 * Update the DBM group file with the new entry as well.
-	 */
-
-	if (is_shadow_grp && sg_dbm_present () && !sg_dbm_update (&sgrp)) {
-		fprintf (stderr, _("%s: cannot add new dbm group entry\n"),
-			 Prog);
-		fail_exit (E_GRP_UPDATE);
-	}
-	endsgent ();
-#endif				/* NDBM */
 #endif				/* SHADOWGRP */
-	SYSLOG ((LOG_INFO, "new group: name=%s, gid=%u",
+	SYSLOG ((LOG_INFO, "new group: name=%s, GID=%u",
 		 group_name, (unsigned int) group_id));
 }
 
@@ -268,7 +230,7 @@ static void find_new_gid (void)
 #endif
 				continue;
 			}
-			fprintf (stderr, _("%s: gid %u is not unique\n"),
+			fprintf (stderr, _("%s: GID %u is not unique\n"),
 				 Prog, (unsigned int) group_id);
 			fail_exit (E_GID_IN_USE);
 		}
@@ -292,8 +254,7 @@ static void find_new_gid (void)
 #endif
 		}
 		if (group_id == gid_max) {
-			fprintf (stderr, _("%s: can't get unique gid\n"),
-				 Prog);
+			fprintf (stderr, _("%s: can't get unique GID\n"), Prog);
 			fail_exit (E_GID_IN_USE);
 		}
 	}
@@ -334,8 +295,18 @@ static void process_flags (int argc, char **argv)
 	char *cp;
 	int arg;
 
-	while ((arg = getopt (argc, argv, "og:O:f")) != EOF) {
+	while ((arg = getopt (argc, argv, "fg:K:o")) != EOF) {
 		switch (arg) {
+		case 'f':
+			/*
+			 * "force" - do nothing, just exit(0), if the
+			 * specified group already exists. With -g, if
+			 * specified gid already exists, choose another
+			 * (unique) gid (turn off -g). Based on the RedHat's
+			 * patch from shadow-utils-970616-9.
+			 */
+			fflg++;
+			break;
 		case 'g':
 			gflg++;
 			if (!isdigit (optarg[0]))
@@ -349,19 +320,16 @@ static void process_flags (int argc, char **argv)
 				fail_exit (E_BAD_ARG);
 			}
 			break;
-		case 'o':
-			oflg++;
-			break;
-		case 'O':
+		case 'K':
 			/*
-			 * override login.defs defaults (-O name=value)
-			 * example: -O GID_MIN=100 -O GID_MAX=499
-			 * note: -O GID_MIN=10,GID_MAX=499 doesn't work yet
+			 * override login.defs defaults (-K name=value)
+			 * example: -K GID_MIN=100 -K GID_MAX=499
+			 * note: -K GID_MIN=10,GID_MAX=499 doesn't work yet
 			 */
 			cp = strchr (optarg, '=');
 			if (!cp) {
 				fprintf (stderr,
-					 _("%s: -O requires NAME=VALUE\n"),
+					 _("%s: -K requires KEY=VALUE\n"),
 					 Prog);
 				exit (E_BAD_ARG);
 			}
@@ -370,15 +338,8 @@ static void process_flags (int argc, char **argv)
 			if (putdef_str (optarg, cp) < 0)
 				exit (E_BAD_ARG);
 			break;
-		case 'f':
-			/*
-			 * "force" - do nothing, just exit(0), if the
-			 * specified group already exists. With -g, if
-			 * specified gid already exists, choose another
-			 * (unique) gid (turn off -g). Based on the RedHat's
-			 * patch from shadow-utils-970616-9.
-			 */
-			fflg++;
+		case 'o':
+			oflg++;
 			break;
 		default:
 			usage ();
@@ -405,16 +366,14 @@ static void process_flags (int argc, char **argv)
 static void close_files (void)
 {
 	if (!gr_close ()) {
-		fprintf (stderr, _("%s: cannot rewrite group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: cannot rewrite group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 	gr_unlock ();
 #ifdef	SHADOWGRP
 	if (is_shadow_grp && !sgr_close ()) {
 		fprintf (stderr,
-			 _("%s: cannot rewrite shadow group file\n"),
-			 Prog);
+			 _("%s: cannot rewrite shadow group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 	if (is_shadow_grp)
@@ -431,26 +390,22 @@ static void close_files (void)
 static void open_files (void)
 {
 	if (!gr_lock ()) {
-		fprintf (stderr, _("%s: unable to lock group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: unable to lock group file\n"), Prog);
 		exit (E_GRP_UPDATE);
 	}
 	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: unable to open group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: unable to open group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
 	if (is_shadow_grp && !sgr_lock ()) {
 		fprintf (stderr,
-			 _("%s: unable to lock shadow group file\n"),
-			 Prog);
+			 _("%s: unable to lock shadow group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 	if (is_shadow_grp && !sgr_open (O_RDWR)) {
 		fprintf (stderr,
-			 _("%s: unable to open shadow group file\n"),
-			 Prog);
+			 _("%s: unable to open shadow group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 #endif				/* SHADOWGRP */
@@ -499,9 +454,9 @@ int main (int argc, char **argv)
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
-	OPENLOG("groupadd");
+	OPENLOG ("groupadd");
 
-	process_flags(argc, argv);
+	process_flags (argc, argv);
 
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
@@ -512,8 +467,7 @@ int main (int argc, char **argv)
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval =
-		    pam_start ("groupadd", pampw->pw_name, &conv, &pamh);
+		retval = pam_start ("groupadd", pampw->pw_name, &conv, &pamh);
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -531,8 +485,7 @@ int main (int argc, char **argv)
 	}
 
 	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM authentication failed\n"),
-			 Prog);
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (1);
 	}
 
@@ -543,18 +496,6 @@ int main (int argc, char **argv)
 	is_shadow_grp = sgr_file_present ();
 #endif
 
-	/*
-	 * The open routines for the DBM files don't use read-write as the
-	 * mode, so we have to clue them in.
-	 */
-
-#ifdef	NDBM
-	gr_dbm_mode = O_RDWR;
-#ifdef	SHADOWGRP
-	sg_dbm_mode = O_RDWR;
-#endif				/* SHADOWGRP */
-#endif				/* NDBM */
-
 	/*
 	 * Start with a quick check to see if the group exists.
 	 */
@@ -563,8 +504,7 @@ int main (int argc, char **argv)
 		if (fflg) {
 			exit (E_SUCCESS);
 		}
-		fprintf (stderr, _("%s: group %s exists\n"), Prog,
-			 group_name);
+		fprintf (stderr, _("%s: group %s exists\n"), Prog, group_name);
 		exit (E_NAME_IN_USE);
 	}
 
@@ -572,7 +512,6 @@ int main (int argc, char **argv)
 	 * Do the hard stuff - open the files, create the group entries,
 	 * then close and update the files.
 	 */
-
 	open_files ();
 
 	if (!gflg || !oflg)
