@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: login.c,v 1.38 2005/01/17 23:12:04 kloczek Exp $")
+RCSID (PKG_VER "$Id: login.c,v 1.66 2005/07/07 15:32:50 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include <sys/stat.h>
@@ -39,25 +39,11 @@ RCSID (PKG_VER "$Id: login.c,v 1.38 2005/01/17 23:12:04 kloczek Exp $")
 #include <pwd.h>
 #include <grp.h>
 #include <signal.h>
-#if HAVE_LASTLOG_H
 #include <lastlog.h>
-#else
-#include "lastlog_.h"
-#endif
 #include "faillog.h"
 #include "failure.h"
 #include "pwauth.h"
 #include "getdef.h"
-#ifdef RADIUS
-/*
- * Support for RADIUS authentication based on a hacked util-linux login
- * source sent to me by Jon Lewis. Not tested. You need to link login
- * with the radauth.c file (not included here - it doesn't have a clear
- * copyright statement, and I don't want to have problems with Debian
- * putting the whole package in non-free because of this).  --marekm
- */
-#include "radlogin.h"
-#endif
 #ifdef UT_ADDR
 #include <netdb.h>
 #endif
@@ -72,7 +58,7 @@ static pam_handle_t *pamh = NULL;
 
 #define PAM_FAIL_CHECK if (retcode != PAM_SUCCESS) { \
 	fprintf(stderr,"\n%s\n",pam_strerror(pamh, retcode)); \
-	syslog(LOG_ERR,"%s",pam_strerror(pamh, retcode)); \
+	SYSLOG((LOG_ERR,"%s",pam_strerror(pamh, retcode))); \
 	pam_end(pamh, retcode); exit(1); \
    }
 #define PAM_END { retcode = pam_close_session(pamh,0); \
@@ -92,12 +78,12 @@ const char *hostname = "";
 static struct passwd pwent;
 
 #if HAVE_UTMPX_H
-extern	struct	utmpx	utxent;
+extern struct utmpx utxent;
 struct utmpx failent;
 #else
 struct utmp failent;
 #endif
-extern	struct	utmp	utent;
+extern struct utmp utent;
 
 struct lastlog lastlog;
 static int pflg = 0;
@@ -131,12 +117,6 @@ extern void dolastlog (struct lastlog *, const struct passwd *,
 
 extern char **environ;
 
-#ifndef USE_PAM
-extern int login_access (const char *, const char *);
-#endif
-
-extern void login_fbtab (const char *, uid_t, gid_t);
-
 #ifndef	ALARM
 #define	ALARM	60
 #endif
@@ -145,19 +125,20 @@ extern void login_fbtab (const char *, uid_t, gid_t);
 #define	RETRIES	3
 #endif
 
-#ifndef USE_PAM
-static struct faillog faillog;
-#endif
-
 /* local function prototypes */
 static void usage (void);
 static void setup_tty (void);
 static void check_flags (int, char *const *);
 
 #ifndef USE_PAM
+extern int login_access (const char *, const char *);
+
+static struct faillog faillog;
+
 static void bad_time_notify (void);
 static void check_nologin (void);
 #endif
+
 static void init_env (void);
 static RETSIGTYPE alarm_handler (int);
 
@@ -182,7 +163,6 @@ static void usage (void)
 	exit (1);
 }
 
-
 static void setup_tty (void)
 {
 	TERMIO termio;
@@ -196,35 +176,8 @@ static void setup_tty (void)
 	termio.c_lflag |= ISIG | ICANON | ECHO | ECHOE;
 	termio.c_iflag |= ICRNL;
 
-#if defined(ECHOKE) && defined(ECHOCTL)
-	termio.c_lflag |= ECHOKE | ECHOCTL;
-#endif
-#if defined(ECHOPRT) && defined(NOFLSH) && defined(TOSTOP)
-	termio.c_lflag &= ~(ECHOPRT | NOFLSH | TOSTOP);
-#endif
-#ifdef	ONLCR
-	termio.c_oflag |= ONLCR;
-#endif
-
-#ifdef	SUN4
-
-	/*
-	 * Terminal setup for SunOS 4.1 courtesy of Steve Allen
-	 * at UCO/Lick.
-	 */
-
-	termio.c_cc[VEOF] = '\04';
-	termio.c_cflag &= ~CSIZE;
-	termio.c_cflag |= (PARENB | CS7);
-	termio.c_lflag |= (ISIG | ICANON | ECHO | IEXTEN);
-	termio.c_iflag |=
-	    (BRKINT | IGNPAR | ISTRIP | IMAXBEL | ICRNL | IXON);
-	termio.c_iflag &= ~IXANY;
-	termio.c_oflag |= (XTABS | OPOST | ONLCR);
-#endif
 	/* leave these values unchanged if not specified in login.defs */
-	termio.c_cc[VERASE] =
-	    getdef_num ("ERASECHAR", termio.c_cc[VERASE]);
+	termio.c_cc[VERASE] = getdef_num ("ERASECHAR", termio.c_cc[VERASE]);
 	termio.c_cc[VKILL] = getdef_num ("KILLCHAR", termio.c_cc[VKILL]);
 
 	/*
@@ -236,48 +189,16 @@ static void setup_tty (void)
 }
 
 
+#ifndef USE_PAM
 /*
  * Tell the user that this is not the right time to login at this tty
  */
-#ifndef USE_PAM
 static void bad_time_notify (void)
 {
-#ifdef HUP_MESG_FILE
-	FILE *mfp;
-
-	if ((mfp = fopen (HUP_MESG_FILE, "r")) != NULL) {
-		int c;
-
-		while ((c = fgetc (mfp)) != EOF) {
-			if (c == '\n')
-				putchar ('\r');
-			putchar (c);
-		}
-		fclose (mfp);
-	} else
-#endif
-		printf (_("Invalid login time\n"));
+	printf (_("Invalid login time\n"));
 	fflush (stdout);
 }
-#endif
 
-
-static void check_flags (int argc, char *const *argv)
-{
-	int arg;
-
-	/*
-	 * Check the flags for proper form. Every argument starting with
-	 * "-" must be exactly two characters long. This closes all the
-	 * clever rlogin, telnet, and getty holes.
-	 */
-	for (arg = 1; arg < argc; arg++) {
-		if (argv[arg][0] == '-' && strlen (argv[arg]) > 2)
-			usage ();
-	}
-}
-
-#ifndef USE_PAM
 static void check_nologin (void)
 {
 	char *fname;
@@ -310,8 +231,7 @@ static void check_nologin (void)
 			fflush (stdout);
 			fclose (nlfp);
 		} else
-			printf (_
-				("\nSystem closed for routine maintenance\n"));
+			printf (_("\nSystem closed for routine maintenance\n"));
 		/*
 		 * Non-root users must exit. Root gets the message, but
 		 * gets to login.
@@ -321,12 +241,27 @@ static void check_nologin (void)
 			closelog ();
 			exit (0);
 		}
-		printf (_
-			("\n[Disconnect bypassed -- root login allowed.]\n"));
+		printf (_("\n[Disconnect bypassed -- root login allowed.]\n"));
 	}
 }
 #endif				/* !USE_PAM */
 
+static void check_flags (int argc, char *const *argv)
+{
+	int arg;
+
+	/*
+	 * Check the flags for proper form. Every argument starting with
+	 * "-" must be exactly two characters long. This closes all the
+	 * clever rlogin, telnet, and getty holes.
+	 */
+	for (arg = 1; arg < argc; arg++) {
+		if (argv[arg][0] == '-' && strlen (argv[arg]) > 2)
+			usage ();
+	}
+}
+
+
 static void init_env (void)
 {
 	char *cp, *tmp;
@@ -342,9 +277,11 @@ static void init_env (void)
 
 	if ((tmp = getenv ("TZ"))) {
 		addenv ("TZ", tmp);
-	} else if ((cp = getdef_str ("ENV_TZ")))
+	} 
+#ifndef USE_PAM
+		else if ((cp = getdef_str ("ENV_TZ")))
 		addenv (*cp == '/' ? tz (cp) : cp, NULL);
-
+#endif /* !USE_PAM */
 	/* 
 	 * Add the clock frequency so that profiling commands work
 	 * correctly.
@@ -352,15 +289,17 @@ static void init_env (void)
 
 	if ((tmp = getenv ("HZ"))) {
 		addenv ("HZ", tmp);
-	} else if ((cp = getdef_str ("ENV_HZ")))
+	} 
+#ifndef USE_PAM
+		else if ((cp = getdef_str ("ENV_HZ")))
 		addenv (cp, NULL);
+#endif /* !USE_PAM */
 }
 
 
 static RETSIGTYPE alarm_handler (int sig)
 {
-	fprintf (stderr, _("\nLogin timed out after %d seconds.\n"),
-		 timeout);
+	fprintf (stderr, _("\nLogin timed out after %d seconds.\n"), timeout);
 	exit (0);
 }
 
@@ -413,26 +352,9 @@ int main (int argc, char **argv)
 	int retcode;
 	pid_t child;
 	char *pam_user;
-#endif				/* USE_PAM */
-#if defined(SHADOWPWD) && !defined(USE_PAM)
+#else
 	struct spwd *spwd = NULL;
 #endif
-#ifdef RADIUS
-	RAD_USER_DATA rad_user_data;
-	int is_rad_login;
-#endif
-#if defined(RADIUS) || defined(DES_RPC) || defined(KERBEROS)
-	/* from pwauth.c */
-	extern char *clear_pass;
-	extern int wipe_clear_pass;
-
-	/*
-	 * We may need the password later, don't want pw_auth() to wipe it
-	 * (we do it ourselves when it is no longer needed).  --marekm
-	 */
-	wipe_clear_pass = 0;
-#endif
-
 	/*
 	 * Some quick initialization.
 	 */
@@ -451,20 +373,30 @@ int main (int argc, char **argv)
 
 	check_flags (argc, argv);
 
-	while ((flag = getopt (argc, argv, "d:f:h:pr:")) != EOF) {
+	while ((flag = getopt (argc, argv, "d:f::h:pr:")) != EOF) {
 		switch (flag) {
-		case 'p':
-			pflg++;
+		case 'd':
+			/* "-d device" ignored for compatibility */
 			break;
 		case 'f':
 			/*
 			 * username must be a separate token
 			 * (-f root, *not* -froot).  --marekm
+			 *
+			 * if -f has an arg, use that, else use the
+			 * normal user name passed after all options
+			 * --benc
 			 */
-			if (optarg != argv[optind - 1])
+			if (optarg != NULL && optarg != argv[optind - 1])
 				usage ();
 			fflg++;
-			STRFCPY (username, optarg);
+			if (optarg)
+				STRFCPY (username, optarg);
+			break;
+		case 'h':
+			hflg++;
+			hostname = optarg;
+			reason = PW_TELNET;
 			break;
 #ifdef	RLOGIN
 		case 'r':
@@ -473,13 +405,8 @@ int main (int argc, char **argv)
 			reason = PW_RLOGIN;
 			break;
 #endif
-		case 'h':
-			hflg++;
-			hostname = optarg;
-			reason = PW_TELNET;
-			break;
-		case 'd':
-			/* "-d device" ignored for compatibility */
+		case 'p':
+			pflg++;
 			break;
 		default:
 			usage ();
@@ -500,7 +427,7 @@ int main (int argc, char **argv)
 	 */
 
 	if ((rflg || fflg || hflg) && !amroot) {
-		fprintf (stderr, _("%s: permission denied.\n"), Prog);
+		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 		exit (1);
 	}
 
@@ -541,8 +468,7 @@ int main (int argc, char **argv)
 		 * be unknown, etc.).  --marekm
 		 */
 		if ((he = gethostbyname (hostname))) {
-			utent.ut_addr =
-			    *((int32_t *) (he->h_addr_list[0]));
+			utent.ut_addr = *((int32_t *) (he->h_addr_list[0]));
 #endif
 #ifdef UT_HOST
 			strncpy (utent.ut_host, hostname,
@@ -581,6 +507,7 @@ int main (int argc, char **argv)
 
 		setup_tty ();
 
+#ifndef USE_PAM
 		umask (getdef_num ("UMASK", 077));
 
 		{
@@ -597,6 +524,7 @@ int main (int argc, char **argv)
 				set_filesize_limit (limit);
 		}
 
+#endif
 		/*
 		 * The entire environment will be preserved if the -p flag
 		 * is used.
@@ -618,7 +546,7 @@ int main (int argc, char **argv)
 		init_env ();
 
 		if (optind < argc) {	/* get the user name */
-			if (rflg || fflg)
+			if (rflg || (fflg && username[0]))
 				usage ();
 
 			STRFCPY (username, argv[optind]);
@@ -645,8 +573,7 @@ int main (int argc, char **argv)
 
 		if (*cp)
 			snprintf (fromhost, sizeof fromhost,
-				  _(" on `%.100s' from `%.200s'"), tty,
-				  cp);
+				  _(" on `%.100s' from `%.200s'"), tty, cp);
 		else
 			snprintf (fromhost, sizeof fromhost,
 				  _(" on `%.100s'"), tty);
@@ -668,8 +595,8 @@ int main (int argc, char **argv)
 			fprintf (stderr,
 				 "login: PAM Failure, aborting: %s\n",
 				 pam_strerror (pamh, retcode));
-			syslog (LOG_ERR, "Couldn't initialize PAM: %s",
-				pam_strerror (pamh, retcode));
+			SYSLOG ((LOG_ERR, "Couldn't initialize PAM: %s",
+				pam_strerror (pamh, retcode)));
 			exit (99);
 		}
 		/*
@@ -698,12 +625,10 @@ int main (int argc, char **argv)
 					  "%s login: ", hostn);
 			else
 				snprintf (login_prompt,
-					  sizeof (login_prompt),
-					  "login: ");
+					  sizeof (login_prompt), "login: ");
 
 			retcode =
-			    pam_set_item (pamh, PAM_USER_PROMPT,
-					  login_prompt);
+			    pam_set_item (pamh, PAM_USER_PROMPT, login_prompt);
 			PAM_FAIL_CHECK;
 
 			/* if we didn't get a user on the command line,
@@ -730,10 +655,10 @@ int main (int argc, char **argv)
 				(retcode == PAM_AUTHINFO_UNAVAIL))) {
 				pam_get_item (pamh, PAM_USER,
 					      (const void **) &pam_user);
-				syslog (LOG_NOTICE,
+				SYSLOG ((LOG_NOTICE,
 					"FAILED LOGIN %d FROM %s FOR %s, %s",
 					failcount, hostname, pam_user,
-					pam_strerror (pamh, retcode));
+					pam_strerror (pamh, retcode)));
 #ifdef HAVE_PAM_FAIL_DELAY
 				pam_fail_delay (pamh, 1000000 * delay);
 #endif
@@ -747,18 +672,16 @@ int main (int argc, char **argv)
 					      (const void **) &pam_user);
 
 				if (retcode == PAM_MAXTRIES)
-					syslog (LOG_NOTICE,
+					SYSLOG ((LOG_NOTICE,
 						"TOO MANY LOGIN TRIES (%d) FROM %s FOR %s, %s",
 						failcount, hostname,
 						pam_user,
-						pam_strerror (pamh,
-							      retcode));
+						pam_strerror (pamh, retcode)));
 				else
-					syslog (LOG_NOTICE,
+					SYSLOG ((LOG_NOTICE,
 						"FAILED LOGIN SESSION FROM %s FOR %s, %s",
 						hostname, pam_user,
-						pam_strerror (pamh,
-							      retcode));
+						pam_strerror (pamh, retcode)));
 
 				fprintf (stderr, "\nLogin incorrect\n");
 				pam_end (pamh, retcode);
@@ -780,51 +703,40 @@ int main (int argc, char **argv)
 		   First get the username that we are actually using, though.
 		 */
 		retcode =
-		    pam_get_item (pamh, PAM_USER,
-				  (const void **) &pam_user);
+		    pam_get_item (pamh, PAM_USER, (const void **) &pam_user);
 		setpwent ();
 		pwd = getpwnam (pam_user);
 
 		if (fflg) {
-			retcode = pam_acct_mgmt(pamh, 0);
+			retcode = pam_acct_mgmt (pamh, 0);
 			PAM_FAIL_CHECK;
 		}
 
 		if (!pwd || setup_groups (pwd))
 			exit (1);
+		else
+			pwent = *pwd;
 
 		retcode = pam_setcred (pamh, PAM_ESTABLISH_CRED);
 		PAM_FAIL_CHECK;
 
-		retcode = pam_open_session (pamh, 0);
+		retcode = pam_open_session (pamh,
+					    hushed (&pwent) ? PAM_SILENT : 0);
 		PAM_FAIL_CHECK;
 
 #else				/* ! USE_PAM */
 		while (1) {	/* repeatedly get login/password pairs */
 			failed = 0;	/* haven't failed authentication yet */
-#ifdef RADIUS
-			is_rad_login = 0;
-#endif
-		if (!username[0]) {	/* need to get a login id */
-			if (subroot) {
-				closelog ();
-				exit (1);
+			if (!username[0]) {	/* need to get a login id */
+				if (subroot) {
+					closelog ();
+					exit (1);
+				}
+				preauth_flag = 0;
+				login_prompt (_("\n%s login: "), username,
+					      sizeof username);
+				continue;
 			}
-			preauth_flag = 0;
-#ifndef LOGIN_PROMPT
-#ifdef __linux__		/* hostname login: - like in util-linux login */
-			login_prompt (_("\n%s login: "), username,
-				      sizeof username);
-#else
-			login_prompt (_("login: "), username,
-				      sizeof username);
-#endif
-#else
-			login_prompt (LOGIN_PROMPT, username,
-				      sizeof username);
-#endif
-			continue;
-		}
 #endif				/* ! USE_PAM */
 
 #ifdef USE_PAM
@@ -844,11 +756,8 @@ int main (int argc, char **argv)
 			pwent = *pwd;
 		}
 #ifndef USE_PAM
-#ifdef SHADOWPWD
 		spwd = NULL;
-		if (pwd
-		    && strcmp (pwd->pw_passwd,
-			       SHADOW_PASSWD_STRING) == 0) {
+		if (pwd && strcmp (pwd->pw_passwd, SHADOW_PASSWD_STRING) == 0) {
 			spwd = getspnam (username);
 			if (spwd)
 				pwent.pw_passwd = spwd->sp_pwdp;
@@ -857,15 +766,13 @@ int main (int argc, char **argv)
 					 "no shadow password for `%s'%s",
 					 username, fromhost));
 		}
-#endif				/* SHADOWPWD */
 
 		/*
 		 * If the encrypted password begins with a "!", the account
 		 * is locked and the user cannot login, even if they have
 		 * been "pre-authenticated."
 		 */
-		if (pwent.pw_passwd[0] == '!'
-		    || pwent.pw_passwd[0] == '*')
+		if (pwent.pw_passwd[0] == '!' || pwent.pw_passwd[0] == '*')
 			failed = 1;
 
 		/*
@@ -876,30 +783,9 @@ int main (int argc, char **argv)
 			goto auth_ok;
 
 		if (pw_auth
-		    (pwent.pw_passwd, username, reason,
-		     (char *) 0) == 0)
+		    (pwent.pw_passwd, username, reason, (char *) 0) == 0)
 			goto auth_ok;
 
-#ifdef RADIUS
-		/*
-		 * If normal passwd authentication didn't work, try radius.
-		 */
-
-		if (failed) {
-			pwd =
-			    rad_authenticate (&rad_user_data,
-					      username,
-					      clear_pass ?
-					      clear_pass : "");
-			if (pwd) {
-				is_rad_login = 1;
-				pwent = *pwd;
-				failed = 0;
-				goto auth_ok;
-			}
-		}
-#endif				/* RADIUS */
-
 		/*
 		 * Don't log unknown usernames - I mistyped the password for
 		 * username at least once. Should probably use LOG_AUTHPRIV
@@ -918,32 +804,20 @@ int main (int argc, char **argv)
 		 * authenticated and so on.
 		 */
 
-#if defined(RADIUS) && !(defined(DES_RPC) || defined(KERBEROS))
-		if (clear_pass) {
-			strzero (clear_pass);
-			clear_pass = NULL;
-		}
-#endif
-
 		if (!failed && pwent.pw_name && pwent.pw_uid == 0
 		    && !is_console) {
-			SYSLOG ((LOG_CRIT, "ILLEGAL ROOT LOGIN %s",
-				 fromhost));
+			SYSLOG ((LOG_CRIT, "ILLEGAL ROOT LOGIN %s", fromhost));
 			failed = 1;
 		}
-#ifdef LOGIN_ACCESS
 		if (!failed
-		    && !login_access (username,
-		      *hostname ? hostname : tty))
-		{
-			SYSLOG ((LOG_WARN, "LOGIN `%s' REFUSED %s", username,
-				 fromhost));
+		    && !login_access (username, *hostname ? hostname : tty)) {
+			SYSLOG ((LOG_WARN, "LOGIN `%s' REFUSED %s",
+				 username, fromhost));
 			failed = 1;
 		}
-#endif
 		if (pwd && getdef_bool ("FAILLOG_ENAB") &&
 		    !failcheck (pwent.pw_uid, &faillog, failed)) {
-			SYSLOG ((LOG_CRIT, 
+			SYSLOG ((LOG_CRIT,
 				 "exceeded failure limit for `%s' %s",
 				 username, fromhost));
 			failed = 1;
@@ -960,14 +834,15 @@ int main (int argc, char **argv)
 #if HAVE_UTMPX_H
 			failent = utxent;
 			if (sizeof (failent.ut_tv) == sizeof (struct timeval))
-			  gettimeofday ((struct timeval *) &failent.ut_tv, NULL);
-			else
-			  {
+				gettimeofday ((struct timeval *)
+					      &failent.ut_tv, NULL);
+			else {
 				struct timeval tv;
+
 				gettimeofday (&tv, NULL);
 				failent.ut_tv.tv_sec = tv.tv_sec;
 				failent.ut_tv.tv_usec = tv.tv_usec;
-			  }
+			}
 #else
 			failent = utent;
 			failent.ut_time = time (NULL);
@@ -975,8 +850,7 @@ int main (int argc, char **argv)
 			if (pwd) {
 				failent_user = pwent.pw_name;
 			} else {
-				if (getdef_bool
-				    ("LOG_UNKFAIL_ENAB"))
+				if (getdef_bool ("LOG_UNKFAIL_ENAB"))
 					failent_user = username;
 				else
 					failent_user = "UNKNOWN";
@@ -1003,8 +877,7 @@ int main (int argc, char **argv)
 		 */
 
 		if (pwent.pw_passwd[0] == '\0')
-			pw_auth ("!", username, reason,
-				 (char *) 0);
+			pw_auth ("!", username, reason, (char *) 0);
 #endif
 		/*
 		 * Wait a while (a la SVR4 /usr/bin/login) before attempting
@@ -1022,9 +895,9 @@ int main (int argc, char **argv)
 			closelog ();
 			exit (1);
 		}
-	}		/* while (1) */
+	}			/* while (1) */
 #endif				/* ! USE_PAM */
-	alarm (0);	/* turn off alarm clock */
+	alarm (0);		/* turn off alarm clock */
 #ifndef USE_PAM			/* PAM does this */
 	/*
 	 * porttime checks moved here, after the user has been
@@ -1057,9 +930,7 @@ int main (int argc, char **argv)
 		subroot++;	/* say i was here again */
 		endpwent ();	/* close all of the file which were */
 		endgrent ();	/* open in the original rooted file */
-#ifdef	SHADOWPWD
 		endspent ();	/* system. they will be re-opened */
-#endif
 #ifdef	SHADOWGRP
 		endsgent ();	/* in the new rooted file system */
 #endif
@@ -1067,8 +938,7 @@ int main (int argc, char **argv)
 	}
 #ifndef USE_PAM			/* pam_lastlog handles this */
 	if (getdef_bool ("LASTLOG_ENAB"))	/* give last login and log this one */
-		dolastlog (&lastlog, &pwent, utent.ut_line,
-			   hostname);
+		dolastlog (&lastlog, &pwent, utent.ut_line, hostname);
 #endif
 
 #ifndef USE_PAM			/* PAM handles this as well */
@@ -1078,8 +948,7 @@ int main (int argc, char **argv)
 	 * and changes to the user in the child before executing the passwd
 	 * program.  --marekm
 	 */
-#ifdef	SHADOWPWD
-	if (spwd) {	/* check for age of password */
+	if (spwd) {		/* check for age of password */
 		if (expire (&pwent, spwd)) {
 			pwd = getpwnam (username);
 			spwd = getspnam (username);
@@ -1087,60 +956,44 @@ int main (int argc, char **argv)
 				pwent = *pwd;
 		}
 	}
-#endif				/* SHADOWPWD */
-
-#ifdef RADIUS
-	if (is_rad_login) {
-		char whofilename[128];
-		FILE *whofile;
-
-		snprintf (whofilename, sizeof whofilename,
-			  "/var/log/radacct/%.20s", tty);
-		whofile = fopen (whofilename, "w");
-		if (whofile) {
-			fprintf (whofile, "%s\n", username);
-			fclose (whofile);
-		}
-	}
-#endif
 	setup_limits (&pwent);	/* nice, ulimit etc. */
 #endif				/* ! USE_PAM */
 	chown_tty (tty, &pwent);
 
-#ifdef LOGIN_FBTAB
+#ifdef USE_PAM
 	/*
-	 * XXX - not supported yet. Change permissions and ownerships of
-	 * devices like floppy/audio/mouse etc. for console logins, based
-	 * on /etc/fbtab or /etc/logindevperm configuration files (Suns do
-	 * this with their framebuffer devices). Problems:
+	 * We must fork before setuid() because we need to call
+	 * pam_close_session() as root.
 	 *
-	 * - most systems (except BSD) don't have that nice revoke() system
-	 *   call to ensure the previous user didn't leave a process holding
-	 *   one of these devices open or mmap'ed. Any volunteers to do it
-	 *   in Linux?
-	 *
-	 * - what to do with different users logged in on different virtual
-	 *   consoles?  Maybe permissions should be changed only on user's
-	 *   request, by running a separate (setuid root) program?
-	 *
-	 * - init/telnetd/rlogind/whatever should restore permissions after
-	 *   the user logs out.
-	 *
-	 * Try the new CONSOLE_GROUPS feature instead. It adds specified
-	 * groups (like "floppy") to the group set if the user is logged in
-	 * on the console. This still has the first problem (users leaving
-	 * processes with these devices open), but doesn't need to change
-	 * any permissions, just make them 0660 root:floppy etc.  --marekm
-	 *
-	 * Warning: users can still gain permanent access to these groups
-	 * unless any user-writable filesystems are mounted with the
-	 * "nosuid" option. Alternatively, the kernel could be modified to
-	 * prevent ordinary users from setting the setgid bit on
-	 * executables.
+	 * Note: not true in other (non-Linux) PAM implementations, where
+	 * the parent process of login (init, telnetd, ...) is responsible
+	 * for calling pam_close_session(). This avoids an extra process for
+	 * each login. Maybe we should do this on Linux too? We let the
+	 * admin configure whether they need to keep login around to close
+	 * sessions.
 	 */
-	login_fbtab (tty, pwent.pw_uid, pwent.pw_gid);
+	if (getdef_bool ("CLOSE_SESSIONS")) {
+		signal (SIGINT, SIG_IGN);
+		child = fork ();
+		if (child < 0) {
+			/* error in fork() */
+			fprintf (stderr,
+				 "login: failure forking: %s",
+				 strerror (errno));
+			PAM_END;
+			exit (0);
+		} else if (child) {
+			/*
+			 * parent - wait for child to finish, then cleanup
+			 * session
+			 */
+			wait (NULL);
+			PAM_END;
+			exit (0);
+		}
+		/* child */
+	}
 #endif
-
 	/* We call set_groups() above because this clobbers pam_groups.so */
 #ifndef USE_PAM
 	if (setup_uid_gid (&pwent, is_console))
@@ -1149,19 +1002,6 @@ int main (int argc, char **argv)
 #endif
 		exit (1);
 
-#ifdef KERBEROS
-	if (clear_pass)
-		login_kerberos (username, clear_pass);
-#endif
-#ifdef DES_RPC
-	if (clear_pass)
-		login_desrpc (clear_pass);
-#endif
-#if defined(DES_RPC) || defined(KERBEROS)
-	if (clear_pass)
-		strzero (clear_pass);
-#endif
-
 	setup_env (&pwent);	/* set env vars, cd to the home dir */
 
 #ifdef USE_PAM
@@ -1198,13 +1038,13 @@ int main (int argc, char **argv)
 				      ("Warning: login re-enabled after temporary lockout.\n"));
 				SYSLOG ((LOG_WARN,
 					 "login `%s' re-enabled after temporary lockout (%d failures)",
-					 username,
-					 (int) faillog.fail_cnt));
+					 username, (int) faillog.fail_cnt));
 			}
 		}
 		if (getdef_bool ("LASTLOG_ENAB")
 		    && lastlog.ll_time != 0) {
 			time_t ll_time = lastlog.ll_time;
+
 #ifdef HAVE_STRFTIME
 			strftime (ptime, sizeof (ptime),
 				  "%a %b %e %H:%M:%S %z %Y",
@@ -1213,8 +1053,7 @@ int main (int argc, char **argv)
 				ptime, lastlog.ll_line);
 #else
 			printf (_("Last login: %.19s on %s"),
-				ctime (&ll_time),
-				lastlog.ll_line);
+				ctime (&ll_time), lastlog.ll_line);
 #endif
 #ifdef HAVE_LL_HOST		/* __linux__ || SUN4 */
 			if (lastlog.ll_host[0])
@@ -1224,82 +1063,37 @@ int main (int argc, char **argv)
 #endif
 			printf (".\n");
 		}
-#ifdef	SHADOWPWD
 		agecheck (&pwent, spwd);
-#else
-		agecheck (&pwent);
-#endif
 
 		mailcheck ();	/* report on the status of mail */
 #endif				/* !USE_PAM */
 	} else
 		addenv ("HUSHLOGIN=TRUE", NULL);
 
-	if (getdef_str ("TTYTYPE_FILE") != NULL
-	    && getenv ("TERM") == NULL)
+	if (getdef_str ("TTYTYPE_FILE") != NULL && getenv ("TERM") == NULL)
 		ttytype (tty);
 
 	signal (SIGQUIT, SIG_DFL);	/* default quit signal */
 	signal (SIGTERM, SIG_DFL);	/* default terminate signal */
 	signal (SIGALRM, SIG_DFL);	/* default alarm signal */
 	signal (SIGHUP, SIG_DFL);	/* added this.  --marekm */
-
-#ifdef USE_PAM
-	/*
-	 * We must fork before setuid() because we need to call
-	 * pam_close_session() as root.
-	 *
-	 * Note: not true in other (non-Linux) PAM implementations, where
-	 * the parent process of login (init, telnetd, ...) is responsible
-	 * for calling pam_close_session(). This avoids an extra process for
-	 * each login. Maybe we should do this on Linux too? We let the
-	 * admin configure whether they need to keep login around to close
-	 * sessions.
-	 */
-	if (getdef_bool ("CLOSE_SESSIONS")) {
-		signal (SIGINT, SIG_IGN);
-		child = fork ();
-		if (child < 0) {
-			/* error in fork() */
-			fprintf (stderr,
-				 "login: failure forking: %s",
-				 strerror (errno));
-			PAM_END;
-			exit (0);
-		} else if (child) {
-			/*
-			 * parent - wait for child to finish, then cleanup
-			 * session
-			 */
-			wait (NULL);
-			PAM_END;
-			exit (0);
-		}
-		/* child */
-	}
-#endif
 	signal (SIGINT, SIG_DFL);	/* default interrupt signal */
 
-	endpwent ();	/* stop access to password file */
-	endgrent ();	/* stop access to group file */
-#ifdef	SHADOWPWD
-	endspent ();	/* stop access to shadow passwd file */
-#endif
+	endpwent ();		/* stop access to password file */
+	endgrent ();		/* stop access to group file */
+	endspent ();		/* stop access to shadow passwd file */
 #ifdef	SHADOWGRP
-	endsgent ();	/* stop access to shadow group file */
+	endsgent ();		/* stop access to shadow group file */
 #endif
 	if (pwent.pw_uid == 0)
 		SYSLOG ((LOG_NOTICE, "ROOT LOGIN %s", fromhost));
 	else if (getdef_bool ("LOG_OK_LOGINS"))
+#ifdef USE_PAM
+		SYSLOG ((LOG_INFO, "`%s' logged in %s", pam_user, fromhost));
+#else
 		SYSLOG ((LOG_INFO, "`%s' logged in %s", username, fromhost));
-	closelog ();
-#ifdef RADIUS
-	if (is_rad_login) {
-		printf (_("Starting rad_login\n"));
-		rad_login (&rad_user_data);
-		exit (0);
-	}
 #endif
+	closelog ();
 	if ((tmp = getdef_str ("FAKE_SHELL")) != NULL) {
 		shell (tmp, pwent.pw_shell);	/* fake shell */
 	}
